/* $Id: ccdcamtr.h,v 1.8 2007/07/23 08:38:43 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_CCDCAMTR_INCLUDED
#define DEP_CCDCAMTR_INCLUDED

#include "ccdcam.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** CCD frame camera class.
 *  Special class for CCD frame cameras for Z/I DMC camera calibration.
 *
 *  @author Dirk Stallmann
 */
class CCDCameraTR : public CCDCamera
{
public:
	/// read
	bool read(istream& is);
	
	/// get camera type
	virtual string type_str() const;
	
	/// get orientation type 
	virtual string orientation_str() const;

	/**@name external/image coordinates transformation
	 *  To convert between external image (pixel) and
	 *  (internal) image coordinates.
	 */
	//@{
	 	/// external-to-image coordinates transformation
		virtual void external_to_image(const DPoint2& ext_c,
			DPoint2& img_c) const;

		/// image-to-external-coords. transformation
		virtual void image_to_external(const DPoint2& img_c,
			DPoint2& ext_c) const;
	//@}
	
	/** transformation of standard deviations
	 *  from external to image coordinates
	 */
	virtual void sdev_external_to_image(const DPoint2& ext_s,
		DPoint2& img_s) const;

};


// non-member functions ...

} // namespace DEP

#endif
