/* $Id: extori3.h,v 1.7 2007/07/23 09:39:29 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_EXTORI3_INCLUDED
#define DEP_EXTORI3_INCLUDED

#include "extori.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Exterior orientation using 3 rotation parameters class.
 *  Intermediate class for classes using 3 rotation parameters,
 *  e.g. Euler angles, Rodrigues parameter.
 *
 *  @author Dirk Stallmann
 */
class ExtOrientation3 : public ExtOrientation
{
public:
	/// constructor
	ExtOrientation3() : ExtOrientation(3) {}
	/*
	/// set and get rotation parameters
	//@{
		/// set rotation parameters
		void set_rot_params(const DPoint3& a)
		{
			_a[EXTORI_A1] = a.x;
			_a[EXTORI_A2] = a.y;
			_a[EXTORI_A3] = a.z;
		}

		/// get rotation parameters
		virtual void get_rot_params(DPoint3& a) const
		{
			a.x = _a[EXTORI_A1];
			a.y = _a[EXTORI_A2];
			a.z = _a[EXTORI_A3]; 
		}
		
		/// set std. devs.
		void set_sdev_rot_params(const DPoint3& sdev)
		{
			_sa[EXTORI_A1] = sdev.x;
			_sa[EXTORI_A2] = sdev.y;
			_sa[EXTORI_A3] = sdev.z;
		}
	//@}
	*/
	// calculate partial derivatives of the rotation matrix
	//virtual void dr(DRotMatrix3& dR_a1, DRotMatrix3& dR_a2,
	//DRotMatrix3& dR_a3) const = 0;
	
	/// calculate partial derivatives of rotation parameters
	void calc_derivs_rot(DerivSub& deriv,
		double h, double u, double v, double w,
		const DPoint3& d, const DRotMatrix3& M) const;
};


// non-member functions ...

} // namespace DEP

#endif
