/* $Id: lobseqn.cpp,v 1.4 2007/07/23 09:40:39 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "drvsub.h"
#include "lobseqn.h"

namespace DEP
{

// constructor

LinObsEqn::LinObsEqn() : _a(), _l(), _p() { }


// constructor with number of observation equations

LinObsEqn::LinObsEqn(int nr) : _a(), _l(nr), _p(nr) { }


// print linearized observation equations

void LinObsEqn::print(ostream& os)
{
	os << "A sub-matrices:\n";
	vector<DerivSub*>::const_iterator d1;
	for (d1 = _a.begin(); d1 != _a.end(); ++d1)
		(*d1)->print(os);
	os << endl;

	os << "p-vector:\n";
	_p.print(os);
	os << endl;

	os << "l-vector:\n";
	_l.print(os);
	os << endl;
}


// add contributions of the lin. obs. eqns. to the normal equations

bool LinObsEqn::add_contr(Matrix<double>& atpa, Vector<double>& atpl,
double& ltpl) const
{
	//assert(_p.rows() == _l.rows());
	int i, j, k, ii, jj;

	// range-checking missing!!!
	// return false;

	vector<DerivSub*>::const_iterator d1;
	for (d1 = _a.begin(); d1 != _a.end(); ++d1)
		{
		vector<DerivSub*>::const_iterator d2;
		for (d2 = d1; d2 != _a.end(); ++d2)
			{
			for (i = 0, ii = (*d1)->index(); i < (*d1)->cols(); i++, ii++)
				for (j = 0, jj = (*d2)->index(); j < (*d2)->cols(); j++, jj++)
					{
					for (k = 0; k < _p.size(); k++)
						atpa(ii,jj) += _p(k) * (*d1)->elem(k,i)
							* (*d2)->elem(k,j);
					}
			}
		}

	for (d1 = _a.begin(); d1 != _a.end(); ++d1)
		{
		for (i = 0, ii = (*d1)->index(); i < (*d1)->cols(); i++, ii++)
			{
			for (k = 0; k < _p.size(); k++)
				atpl(ii) += _p(k) * (*d1)->elem(k,i) * _l(k);
			}
		}

	for (k = 0; k < _p.size(); k++)
		ltpl += _p(k) * _l(k) * _l(k);

	return true;
}

} // namespace DEP
