/* $Id: poscor.cpp,v 1.11 2007/10/29 11:19:21 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "depvec.h"
#include "poscor.h"
#include "drvsub.h"
#include "image.h"

namespace DEP
{

// constructor

PosCorrection::PosCorrection() : AdjObject(),
_image_ids(),
_a(0), _sa(0), _t0(0.)
{
	// empty
}


// read

bool PosCorrection::read(istream& is)
{
	// read image IDs
	string image_id;
	while (is >> image_id && image_id.compare("stop-dep") != 0)
		_image_ids.push_back(image_id);

	// read number of coeffs.
	int sz = 0;
	is >> sz;
	_a = vector<DPoint3>(sz);
	_sa = vector<DPoint3>(sz);

	_size = 3 * _a.size();

	// read coeff. and std. devs.
	for (int i = 0; i < int(_a.size()); i++)
		{
		is >> _a[i].x >> _sa[i].x;
		is >> _a[i].y >> _sa[i].y;
		is >> _a[i].z >> _sa[i].z;
		}

	_approx_flag = true;

	return true;
}


// write
/*
void PosCorrection::write(ostream& os, SDevLim& sdev_lim)
{
	assert(_a.size() == _sa.size());

	os << id << "   " << name << endl;

	for (int i = 0; i < _a.size(); i++)
		{
		os << form("   %10.4e   %10.4e\n", _a[i].x, _sa[i].x);
		os << form("   %10.4e   %10.4e\n", _a[i].y, _sa[i].y);
		os << form("   %10.4e   %10.4e\n", _a[i].z, _sa[i].z);
		os << endl;
		}

	os << endl;
}
*/


// print

void PosCorrection::print(ostream& os) const
{
	assert(_a.size() == _sa.size());

	Form str4; str4.left().width(4);
	Form str8; str8.left().width(8);
	Form sci12(3); sci12.scientific().width(12);

	os << "Pos. corr. ID         :  " << _id << endl;
	os << "Images using this set :  " << _image_ids.size();
	{
		for (int i = 0; i < int(_image_ids.size()); i++)
			{
			if (i % 4 == 0) os << endl;
			os << "  " << _image_ids[i];
			}
		os << endl;
	os << endl;
	}
	os << "  Param.     State          Value       Std.dev.\n\n";
	for (size_t i = 0, k = 0; i < _a.size(); i++)
		{
		os << "  " << str8(uname(k++)) << "   " << str4(state_str(_sa[i].x))
			<< "   " << sci12(_a[i].x) << "   " << sci12(_sa[i].x) << endl;
		os << "  " << str8(uname(k++)) << "   " << str4(state_str(_sa[i].y))
			<< "   " << sci12(_a[i].y) << "   " << sci12(_sa[i].y) << endl;
		os << "  " << str8(uname(k++)) << "   " << str4(state_str(_sa[i].z))
			<< "   " << sci12(_a[i].z) << "   " << sci12(_sa[i].z) << endl;
		os << endl;
		}
	
	os << endl;
}


// get adjustment object class name

string PosCorrection::class_str() const
{
	ostringstream oss;
	oss << "position correction";
	return oss.str();
}


// get key name

string PosCorrection::key_str() const
{
	ostringstream oss;
	oss << "pos-corr";
	return oss.str();
}


// get parameter name

string PosCorrection::uname(size_t i) const
{
	ostringstream oss;
	
	assert(i < 3 * _a.size());
	oss << "c[";
	switch (i % 3)
		{
		case 0: oss << "Xo"; break;
		case 1: oss << "Yo"; break;
		case 2: oss << "Zo"; break;
		}
	oss << "," << (i / 3) << "]";

	return oss.str();
}


// set upari

bool PosCorrection::set_upari()
{
	if (create() != true) return false;

	int k = 0;
	for (size_t i = 0; i < _a.size(); i++)
		{
		set(k, &_a[i].x, &_sa[i].x); k++;
		set(k, &_a[i].y, &_sa[i].y); k++;
		set(k, &_a[i].z, &_sa[i].z); k++;
		}

	return true;
}


// print unknown parameters
/*
void PosCorrection::print_unknowns(ostream& os) const
{
	os << "Pos. corr. ID: " << _id << endl;
	os << endl;
	os << "  Param.     State         Value       Std.dev.    Ratio\n\n";

	for (int i = 0; i < _num; i++)
		{
		double t_abs = ratio(i); // test value
		os << form("  %-8s   %-4s   %12.3e   %12.3e   %6.1f",
			uname(i), state_cstr(i), par(i), sdev(i), t_abs);

		// test on significance
		if (is_eli(sdev(i)))
			os << "   is eliminated";
		else
			{
			if (chk_sign(t_abs))
				os << "   is significant";
			else
				os << "   not significant";
			}
		os << endl;	
		}
	
	os << endl;	
}
*/


// print changes of unknown parameters
/*
void PosCorrection::print_changes(ostream& os, const Vector<double>& dx) const
{
	//assert(_a.size() == _sa.size());

	os << "Pos. corr. ID    : " << _id << endl;
	os << endl;
	//os << "  Name       State"
	//	"        Value       Std.dev.    Ratio\n\n";

	int k = _index;
	for (int i = 0; i < _num; i++, k++)
		os << form("  %-8s   %-4s   %12.3e   %12.3e   %12.3e\n",
			uname(i), state_cstr(i), par(i), dx(k), (par(i) + dx(k)));
	os << endl;
}
*/


// get position correction
// evaluate polynomial p(dt) with the coefficients a[0..n]

DPoint3 PosCorrection::calc_corr(double t) const
{
	double dt = t - _t0;
	DPoint3 cor = _a[_a.size() - 1];

	for (int j = _a.size() - 2; j >= 0; j--)
		cor = cor * dt + _a[j];
	
	return cor;
}


// attach to images

bool PosCorrection::attach(vector<Image*>& images)
{
	vector<string>::const_iterator it;
	for (it = _image_ids.begin(); it != _image_ids.end(); ++it)
		{
		Image* p = find(images, *it);
		if (p == 0)
			{
			cout << "WARNING: Image '" << *it
				<< "' for pos. correction '"<< _id  << "' not found.\n";
			continue;
			}
		
		p->attach(this);
		this->inuse(true);
		}

	return true;
}


// get derivatives

void PosCorrection::calc_derivs(double t, const DerivSub& deriv_pcc,
DerivSub& deriv_pc) const
{
	int n = 3 * _a.size();
	deriv_pc.index(_index);	// set index to normal equations
	deriv_pc.newsize(2, n);
	
	deriv_pc(0,0) = deriv_pcc(0,0);
	deriv_pc(1,0) = deriv_pcc(1,0);
	deriv_pc(0,1) = deriv_pcc(0,1);
	deriv_pc(1,1) = deriv_pcc(1,1);
	deriv_pc(0,2) = deriv_pcc(0,2);
	deriv_pc(1,2) = deriv_pcc(1,2);

	double dt = t - _t0;

	for (int i = 3; i < n; i++)
		{
		deriv_pc(0,i) = deriv_pc(0,i-3) * dt;
		deriv_pc(1,i) = deriv_pc(1,i-3) * dt;
		}
}

} // namespace DEP
